from .qubopt_common import *

qo.amptreeiface.argtypes = (PNode, POINTER(PNode), QTR_Callback, QTR_Callback, QTR_Callback)
qo.amptreeiface.restype = PNode

def AMP(config, source_segs, on_report=lambda s: None, on_percent=lambda pct: None, main_hold=None, **kw):
    cfg = prep_tree(config)
    for k, v in kw.iteritems():
        cfg[k].data = v
    dat = source_to_dataset(source_segs)
    if main_hold is None:
        def fill_data(seg):
            src = source_segs[0].clone()
            src.f, src.l = seg.data[:2]
            seg['Channel'].data = src.get_samples().samples.astype('float64').flat
            return seg
    else:
        def fill_data(seg):
            with main_hold:
                src = source_segs[0].clone()
                src.f, src.l = seg.data[:2]
                seg['Channel'].data = src.get_samples().samples.astype('float64').reshape((src.l-src.f+1,))
            return seg
    ptr_array = (PNode * 2)()
    ptr_array[0] = dat.impl
    ptr_array[1] = None
    def on_report_node(node):
        on_report(str(node.data))
        return node
    def on_percent_node(node):
        on_percent(node.data[0])
        return node
    result_impl = qo.amptreeiface(cfg.impl, ptr_array, wrap_callback(fill_data), wrap_callback(on_report_node), wrap_callback(on_percent_node))
    result = qubx.tree_native.Node(impl=result_impl)
    qubx.tree_native.qubtree.QTR_DECREF(result_impl)
    return result

def get_amp_config(model, auto_init=False, auto_init_up=True):
    config = qubx.tree_native.Node('Properties')
    mdl = config.append( prep_tree(model.as_tree()) )
    mdl['ChannelCount'].data[0] = 1
    config["AutoInit"].data = int(auto_init)
    config["AutoInitUp"].data = int(auto_init_up)
    config["Idealize"].data = 0
    return config

def source_to_dataset(segs):
    dataset = qubx.tree_native.Node('DataSet')
    dataset['sampling'].data = segs[0].sampling * 1e3 if segs else 1e-1
    dseg = qubx.tree_native.NullNode()
    for seg in segs:
        dseg = dataset.insert('Segment', dseg)
        dseg.data = seg.f, seg.l
        dseg['start'].data = seg.start
    return dataset
